#!/usr/bin/env python3
import falcon 
from falcon.http_error import HTTPError
from oslo_log import log
LOG = log.getLogger(__name__)

class TVEAPIException(falcon.HTTPError):
    message = "unknown error"
    def __init__(self, message=''):
        if message:
            self.message = str(message)
        LOG.error(message)
        HTTPError.__init__(self, falcon.HTTPStatus(400), 
                           "Bad request format", message)

    @staticmethod
    def handle(ex, req, resp, params):
        raise falcon.HTTPError('500 unknown server error',
                               title=_("Internal Server Error"),
                               description=TVEAPIException.message)


class BadDataFormat(TVEAPIException):
    @staticmethod
    def handle(ex, req, resp, params):
        raise falcon.HTTPBadRequest(
            title="Bad request format",
            description=ex.message)
        
class AccessForbidden(TVEAPIException):
    @staticmethod
    def handle(ex, req, resp, params):
        raise falcon.HTTPForbidden(
            title="Access Forbidden",
            description="You are not allowed to access this resource")
        
exception_handlers_catalog = [
    BadDataFormat,
    AccessForbidden,
]
