#!/usr/bin/env python3
# from jsonschema._validators import pattern
#ipr_regext does not work for cases : 0.3.3.3, 2.3.3.0
ip_regex = '(^(((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?))$)|(^$)'
number_regex = '^\\d+$'
hostname_pattern ='^[a-z0-9]{1}[a-z0-9-]{1,61}([a-z0-9])$'
ip_list_regex ='^(((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)(/(1[0-9]|2[0-9]|3[0-2]|[0-9]))?([\s,](?!$)|$))*$'

################## IMAGE/SCRIPTS SERVER ##################
server_properties = {
    "hash-sum": {
        "id": "hash-sum",
        "type": "string"
    },
    "ipv4-address": {
        "id": "ipv4-address",
        "type": "string"
    },
    "password": {
        "id": "password",
        "type": "string"
    },
    "protocol": {
        "id": "protocol",
        "type": "string",
        "enum": ["ftp", "sftp"]
    },
    "user": {
        "id": "username",
        "type": "string"
    },
    "version":{
        "id": "version",
        "type": "string"
    }

}
server_schema = {
    "type": "object",
    "properties": server_properties,
    "additionalProperties": False,
    "required": [ "hash-sum", "ipv4-address", "password", "protocol", "user",
                 "version"],
}

################## CONFIG-SERVER ##################
config_server_schema = {
    "type": "object",
    "properties": {
        "ipv4-address": {
                "id": "ipv4-address",
                "type": "string"
        }
    },
    "additionalProperties": False,
    "required": [ "ipv4-address" ]
}


################## INTERFACE ##################

interface_properties = {
                "type": "object",
                "properties": {
                    "mode":{
                        "id": "mode",
                        "type": "string",
                        "enum": ["virtio","dpdk", "sr-iov"]
                    },
                    "bootproto": {
                        "id": "bootproto",
                        "type": "string"
                    },
                    "dns": {
                        "id": "dns",
                        "type": "string"
                    },
                    "gateway": {
                        "id": "gateway",
                        "type": "string"
                    },
                    "metric": {
                        "id": "metric",
                        "type": "integer"
                    },
                    "trace-host": {
                        "id": "trace-host",
                        "type": "string"
                    },
                    "ipv4-address":{
                        "id": "ipv4-address",
                        "type": "string",
                        "pattern": ip_regex
                    },
                    "ipv6-address":{
                        "id": "ipv6-address",
                        "type": "string",
                    },
                    "management":{
                        "id": "management",
                        "type": "boolean"
                    },
                    "name":{
                        "id": "name",
                        "type":"string"
                    },
                    "netmask":{
                        "id": "netmask",
                        "type":"string"
                    },
                    "port-forwarding-all": {
                        "id": "port-forwarding-all",
                        "type": "boolean"
                    },
                    "type":{
                        "id": "type",
                        "type":"string"
                    },
                    "apn":{
                        "id": "apn",
                        "type":"string"
                    },
                    "pass":{
                        "id": "pass",
                        "type":"string"
                    },
                    "user":{
                        "id": "user",
                        "type":"string"
                    },
                    "enable":{
                        "id": "enable",
                        "type": "boolean"
                    },
                    "vlan":{
                        "id": "vlan",
                        "anyOf": [
                            { "type": "integer", "minimum": 1, "maximum": 4096 },
                            { "type": "string", "maxLength": 0},
                        ]
                    },
                    "wan-interface":{
                        "id": "wan-interface",
                        "type": "array"
                    },
                    "ovpn-cfg": {
                        "concentrator-endpoint": {
                            "id": "concentrator-endpoint",
                            "type": "string",
                        },
                        "concentrator-port": {
                            "id": "concentrator-port",
                            "type": "integer", "minimum": 1, "maximum": 65535
                        }
                    },
                },
                "additionalProperties": False,
            }
interface_properties_req_all = interface_properties.copy()
interface_properties_req_all.update({"required":
                            [  "name", "type" ]})
interfaces_properties_list_required_all = {
    "type": "array",
    "minItems": 1,
    "additionalItems": False,
    "items": interface_properties_req_all
}

interfaces_properties_list = {
    "type": "array",
    "minItems": 1,
    "additionalItems": False,
    "items": interface_properties
}

################## INTERFACES ##################
interfaces_properties = {
            "interfaces": {
                "id": "interfaces",
                "type": "array",
                "minItems": 1,
                "additionalItems": False,
                "items": interface_properties
            }
}
################## NETWORKING ##################
networking_properties = {
    "interfaces": interfaces_properties,
    "routing":{
            "id": "routing",
            "type": "object",
            "properties": {
                "bgp": {
                    "properties": {
                        "neighbor": ""
                    }
                },
               "static":{
                   "properties": ""
            }
        }
    }
}

networking_schema = {
    "type": "object",
    "properties": networking_properties,
    "additionalProperties": True,
    "required": [ "interfaces"],
}

################## NTP ##################
ntp_properties = {
    "server": {
        "id": "server",
        "type": "string"
    },
    "timezone": {
        "id": "timezone",
        "type": "string",
    }
}
ntp_schema = {
    "type": "object",
    "properties": ntp_properties,
    "additionalProperties": False,
    "required": [ "server", "timezone"],
}

################## HOMECALLING ##################
homecalling_properties = {
    "enabled": {
        "id": "enabled",
        "type": "boolean"
    },
    "timeout": {
        "id": "timeout",
        "anyOf": [
            { "type": "integer", "minimum": 60, "maximum": 3600 },
            { "type": "string", "maxLength": 0},
        ]
    }
}

homecalling_schema = {
    "type": "object",
    "properties": homecalling_properties,
    "additionalProperties": False,
    "required": [ "enabled", "timeout"],
}

################## HOSTNAME ##################
hostname_schema = {
        "type": "object",
        "properties": {
            "hostname":{
                "id": "hostname",
                "type": "string",
                "pattern": hostname_pattern
            }
        },
        "additionalProperties": False,
        "required": [ "hostname"],
    }

################## HARDWARE TYPE ##################
hardware_type_schema = {
        "type": "object",
        "properties": {
            "type":{
                "id": "type",
                "type": "string"
            }
        },
        "additionalProperties": False,
        "required": [ "type"]
    }

################## SECURITY AUTHENTICATION##################
security_auth_schema = {
        "type": "object",
        "properties": {
                        "auth-type":{
                            "id": "auth-type",
                            "type": "string",
                            "enum": ["tacacs", "local"]
                        },
                        "tacacs-key":{
                            "id": "tacacs-key",
                            "type": "string",
                        },
                        "tacacs-server":{
                            "id": "tacacs-server",
                            "type": "string",
                        },
        },
        "additionalProperties": False,
        "required": [ "auth-type", "tacacs-key", "tacacs-server"],
}

################## SECURITY CLI-WHITELIST##################
security_cli_whitelist_schema = {
        "type": "object",
        "properties": {
                        "access-source-ip-list":{
                            "id": "access-source-ip-list",
                            "type": "string",
                            "pattern": ip_list_regex
                        },
                        "enabled": {
                            "id": "enabled",
                            "type": "boolean"
                        },
        },
        "additionalProperties": False,
        "required": [ "access-source-ip-list", "enabled"],
}

################## Remote-Servers ##################
remote_server_properties = {
                "type": "object",
                "properties": {
                    "enable": {
                        "id": "enable",
                        "type": "boolean"
                    },
                    "facility": {
                        "id": "facility",
                        "type": "string",
                        "enum": ["auth", "authpriv", "cron", "daemon", "kern",
                                "lpr", "mail", "mark", "news", "syslog", "user",
                                "uucp", "local0", "local1", "local2", "local3",
                                "local4", "local5", "local6", "local7", "*", ""]
                    },
                    "id": {
                        "id": "id",
                        "type": "integer", "minimum": 1, "maximum": 5
                    },
                    "ipv4-address":{ "todo isn't is better the name to ne ipv4-address jst fo conwsistency'"
                        "id": "ipv4-address",
                        "type": "string",
                    },
                    "port":{
                        "id": "port",
                        "type": "integer", "minimum": 1, "maximum": 65535
                    },
                    "severity":{
                        "id": "severity",
                        "type":"string",
                        "enum": ["debug", "info", "notice", "warning", "warn",
                                 "err", "error", "crit", "alert", "emerg", "*", ""]

                    }
                },
                "additionalProperties": False,
                "required": [ "enable", "facility", "id", "ipv4-address",
                            "port", "severity"]
            }

remote_server_properties_list = {
    "type": "array",
    "minItems": 5,
    "maxItems": 5,
    "additionalItems": False,
    "items": remote_server_properties
}



################## SNMP Server ##################
snmp_views_properties = {
                "type": "object",
                "properties": {
                    "id": {
                        "id": "id",
                        "type": "integer", "minimum": 1, "maximum": 2
                    },
                    "name": {
                        "id": "name",
                        "type": "string"
                    },
                    "type": {
                        "id": "type",
                        "type": "string",
                        "enum": ["included", "excluded", ""]
                    },
                    "oid": {
                        "id": "oid",
                        "type": "string"
                    }
                },
                "additionalProperties": False,
                "required": [ "id", "name", "type", "oid"]
            }

snmp_users_properties = {
                "type": "object",
                "properties": {
                    "id": {
                        "id": "id",
                        "type": "integer", "minimum": 1, "maximum": 2
                    },
                    "user-name": {
                        "id": "user-name",
                        "type": "string"
                    },
                    "message-model": {
                        "id": "message-model",
                        "type": "string",
                        "enum": ["v1", "v2c", "v3", ""]
                    },
                    "security-level": {
                        "id": "security-level",
                        "type": "string",
                        "enum": ["authNoPriv", "authPriv", "noAuthNoPriv", ""]
                    },
                    "auth-type": {
                        "id": "auth-type",
                        "type": "string",
                        "enum": [ "MD5", "SHA", "none", ""]
                    },
                    "auth-password": {
                        "id": "auth-password",
                        "type": "string"
                    },
                    "priv-type": {
                        "id": "priv-type",
                        "type": "string",
                        "enum": ["DES", "AES", "none", ""]
                    },
                    "priv-password": {
                        "id": "priv-password",
                        "type": "string"
                    },
                    "remote-engine-id": {
                        "id": "remote-engine-id",
                        "type": "string"
                    },
                    "read-view": {
                        "id": "read-view",
                        "type": "string"
                    },
                    "write-view": {
                        "id": "write-view",
                        "type": "string"
                    },
                    "notify-view": {
                        "id": "notify-view",
                        "type": "string"
                    }
                },
                "additionalProperties": False,
                "required": [ "id", "user-name", "message-model", "security-level",
                             "auth-type", "priv-type", "read-view",
                             "write-view", "notify-view"]
            }

snmp_target_address_properties = {
                "type": "object",
                "properties": {
                    "id": {
                        "id": "id",
                        "type": "integer", "minimum": 1, "maximum": 5
                    },
                    "enable":{
                            "id": "enable",
                            "type": "boolean"
                    },
                    "target-addr-name": {
                        "id": "target-addr-name",
                        "type": "string"
                    },
                    "message-model": {
                        "id": "message-model",
                        "type": "string",
                        "enum": ["v1", "v2c", "v3", ""]
                    },
                    "security-level": {
                        "id": "security-level",
                        "type": "string",
                        "enum": ["authNoPriv", "authPriv", "noAuthNoPriv", ""]
                    },
                    "ipv4-address": {
                        "id": "ipv4-address",
                        "type": "string",
                    },
                    "security-name": {
                        "id": "security-name",
                        "type": "string",
                    },
                    "dst-port": {
                        "id": "dst-port",
                        "type": "integer", "minimum": 1, "maximum": 65535
                    },
                    "timeout": { #applicable only for informs (in milliseconds)
                        "id": "timeout",
                        "type": "integer", "minimum": 0, "maximum": 2147483647
                    },
                    "retry-count": { #applicable only for informs
                        "id": "retry-count",
                        "type": "integer", "minimum": 0, "maximum": 255
                    },
                    "type": {
                        "id": "type",
                        "type": "string",
                        "enum": [ "inform", "trap", ""]
                    },
                },
                "additionalProperties": False,
                "required": ["enable", "id", "target-addr-name", "message-model", "security-level",
                             "ipv4-address", "security-name", "dst-port",
                             "timeout", "retry-count", "type"]
            }

snmp_server_schema = {
        "type": "object",
        "properties": {
                        "enable":{
                            "id": "enable",
                            "type": "boolean"
                        },
                        "bind-address":{
                            "id": "bind-address",
                            "type": "string",
                        },
                        "port": {
                            "id": "port",
                            "type": "integer", "minimum": 1, "maximum": 65535
                        },
                        "authentication-failure-trap-enabled":{
                            "id": "authentication-failure-trap-enabled",
                            "type": "boolean"
                        },
                        "system-name":{
                            "id": "system-name",
                            "type": "string",
                        },
                        "system-location":{
                            "id": "system-location",
                            "type": "string",
                        },
                        "system-contact":{
                            "id": "system-contact",
                            "type": "string",
                        },
                        "system-description":{
                            "id": "system-description",
                            "type": "string",
                        },
                        "views": {
                                "id": "views",
                                "type": "array",
                                "minItems": 2,
                                "maxItems": 2,
                                "additionalItems": False,
                                "items": snmp_views_properties
                        },
                        "users": {
                                "id": "users",
                                "type": "array",
                                "minItems": 2,
                                "maxItems": 2,
                                "additionalItems": False,
                                "items": snmp_users_properties
                        },
                        "target-address": {
                                "id": "target-address",
                                "type": "array",
                                "minItems": 5,
                                "maxItems": 5,
                                "additionalItems": False,
                                "items": snmp_target_address_properties
                        }
        },
        "additionalProperties": False,
        "required": [ "enable", "bind-address", "authentication-failure-trap-enabled",
                     "system-name", "views", "users", "port", "target-address"],
}

################## System-resources ##################
sys_res_schema = {
    "type": "object",
    "properties": {
        "dpdk-reserved-cpus":{
            "id": "dpdk-reserved-cpus",
            "type": "string"
        },
        "host-available-cpus":{
            "id": "host-available-cpus",
            "type": "integer", "minimum": 0, "maximum": 2147483647
        },
        "host-available-memory":{
            "id": "host-available-memory",
            "type": "integer", "minimum": 0, "maximum": 2147483647
        },
        "host-reserved-cpus":{
            "id": "host-reserved-cpus",
            "type": "string"
        },
        "host-reserved-memory":{
            "id": "host-reserved-memory",
            "type": "integer", "minimum": 0, "maximum": 2147483647
        },
        "huge-page-count":{
            "id": "huge-page-count",
            "type": "integer", "minimum": 0, "maximum": 2147483647
        },
        "huge-page-size":{
            "id": "huge-page-size",
            "type": "integer", "minimum": 0, "maximum": 2147483647
        },
        "host-oversubscribe-enabled":{
            "id": "host-oversubscribe-enabled",
            "type": "boolean"
        },
        "light-dpdk":{
            "id": "light-dpdk",
            "type": "boolean"
        }
    },
    "additionalProperties": False,
}

################## DHCP Relay ##################
dhcp_relay_schema = {
    "type": "object",
    "properties": {
        "enabled":{
            "id": "enabled",
            "type": "boolean"
        },
        "dhcp-relay-server":{
            "id": "dhc-relay-server",
            "type": "string"
        },
        "dhcp-relay-interface":{
            "id": "dhc-relay-interface",
            "type": "string"
        }
    },
    "additionalProperties": False,
}

################## DHCP Server ##################
dhcp_server_schema = {
    "type": "object",
    "properties": {
        "enabled":{
            "id": "enabled",
            "type": "boolean"
        },
        "dhcp-range":{
            "id": "dhcp-range",
            "type": "string"
        },
        "dhcp-bind-interface":{
            "id": "dhcp-bind-interface",
            "type": "string"
        }
    },
    "additionalProperties": False,
}

################## Activation Screen Wizard ##################
activation_screen_wizard_schema = {
    "type": "object",
    "properties": {
        "enabled":{
            "id": "enabled",
            "type": "boolean"
        }
    },
    "additionalProperties": False,
}

################## Quagga ##################

quagga_config_properties = {
    "config-server": {
        "id": "config-server",
        "type": "string"
    },
    "username": {
        "id": "username",
        "type": "string",
    },
    "password": {
        "id": "password",
        "type": "string",
    },
    "protocol": {
        "id": "protocol",
        "type": "string",
        "enum": ["ftp", "sftp", ""]
    },
    "filename": {
        "id": "filename",
        "type": "string",
    },
    "hash-sum": {
        "id": "filename",
        "type": "string",
    }
}

quagga_schema = {
    "type": "object",
    "properties": {
        "enabled":{
            "id": "enabled",
            "type": "boolean"
        },
       "zebrad-config": {
                "id": "zebrad-config",
                "type": "object",
                "properties": quagga_config_properties,
                "required": [ "config-server", "username", "password", "protocol",
                             "filename", "hash-sum"],
                "additionalItems": False
        },
        "ospfd-config": {
                "id": "ospfd-config",
                "type": "object",
                "properties": quagga_config_properties,
                "required": [ "config-server", "username", "password", "protocol",
                             "filename", "hash-sum"],
                "additionalItems": False
        },
        "ospf6d-config": {
                "id": "ospf6d-config",
                "type": "object",
                "properties": quagga_config_properties,
                "required": [ "config-server", "username", "password", "protocol",
                             "filename", "hash-sum"],
                "additionalItems": False
        },
        "bgpd-config": {
                "id": "bgpd-config",
                "type": "object",
                "properties": quagga_config_properties,
                "required": [ "config-server", "username", "password", "protocol",
                             "filename", "hash-sum"],
                "additionalItems": False
        }
    },
    "additionalProperties": False,
    "required": [ "enabled", "zebrad-config", "ospfd-config", "ospf6d-config", "bgpd-config"]
}

quagga_enabled_schema = {
        "type": "object",
        "properties": {
            "enabled":{
                "id": "enabled",
                "type": "boolean",
            }
        },
        "additionalProperties": False,
        "required": [ "enabled"],
}

quagga_config_schema = {
    "type": "object",
    "properties": quagga_config_properties,
    "required": [ "config-server", "username", "password", "protocol",
                 "filename", "hash-sum"],
    "additionalItems": False
}

################## Kubernetes DHCP Config ##################
kubernetes_dhcp_config_schema = {
  "type": "object",
  "properties": {
    "dhcp": {
      "type": "object",
      "properties": {
        "dns": {
          "type": "string"
        },
        "domain": {
          "type": "string"
        },
        "workerName": {
          "type": "string"
        },
        "serverAddress": {
          "type": "string"
        },
        "gateway": {
          "type": "string"
        },
        "networkInterface": {
          "type": "string"
        },
        "range": {
          "type": "string"
        },
        "subnet": {
          "type": "string"
        },
        "staticConfigurations": {
          "type": ["array", "null"],
          "items": [
            {
              "type": "object",
              "properties": {
                "fixedIpAddress": {
                  "type": "string"
                },
                "hardwareEthernetAddress": {
                  "type": "string"
                },
                "host": {
                  "type": "string"
                }
              }
            }
          ]
        }
      },
      "required": [
        "dns",
        "domain",
        "gateway",
        "serverAddress",
        "networkInterface",
        "range",
        "subnet"
      ]
    }
  },
  "required": [
    "dhcp"
  ]
}

################## Wireguard Client Config ##################
wireguard_client_config_schema = {
    "type": "object",
    "properties": {
        "concentrator-port": {
         "type": "integer"
        },
        "hostname": {
         "type": "string"
        },
        "password": {
         "type": "string"
        },
        "public-key": {
         "type": "string"
        }
   },
    "required": [
        "concentrator-port",
        "hostname",
        "password",
        "public-key"
    ]
}
wireguard_static_client_config_schema = {
    "type": "object",
    "properties": {
        "hostname": {
         "type": "string"
        },
        "tunnel-ip": {
         "type": "string"
        }
    },
    "required": [
        "hostname",
        "tunnel-ip"
    ]
}
wireguard_client_domain_lookup_config_schema = {
    "type": "object",
    "properties": {
        "hostname": {
         "type": "string"
        },
        "domain-lookup": {
         "type": "boolean"
        }
    },
    "required": [
        "hostname",
        "domain-lookup"
    ]
}
