#!/usr/bin/env python3
import subprocess
from oslo_log import log
from tveapi.common import exceptions
from tveapi import context

LOG = log.getLogger(__name__)
 
def execute_shell_command(command, quiet = False):
    call = subprocess.Popen(command, shell=True,
                            encoding='utf-8', text=True,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE)
    stdoutdata, stderdata = call.communicate()
    if not quiet:
        if call.returncode != 0:
            LOG.error("FAILED command: '{}', returncode : {}, msg: {}, {}"
                      .format(command, call.returncode, stderdata, stdoutdata)) 
        else:
            LOG.info("SUCCESS command: '{}' , returncode : {}, msg: {}, {}"
                      .format(command, call.returncode, stderdata, stdoutdata))
    return (call.returncode, stdoutdata, stderdata)
                
def check_connectivity(host_ip):
    command = 'ping -c 1 %s ' % host_ip
    call = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)
    stdoutdata, stderdata = call.communicate()
    if call.returncode == 0:
        LOG.info(stdoutdata)
    else:
        LOG.error("FAILED command: '{}', returncode : {}, msg: {}, {}"
                .format(command, call.returncode, stderdata, stdoutdata))
    return call.returncode == 0 

def get_bool_value(str_value):
    if str_value.lower() == 'true':
        return True
    elif str_value.lower() == 'false':
        return False
    raise exceptions.TVEAPIException('{} is not a valid Bool value'.format(str_value))

def inject_context(req, resp, params):
# no need to change context here; we change it in ContextMiddleware.get_context
    return
    user_id = req.get_header('X-USER-ID')
    request_id = req.get_header('X-Openstack-Request-ID')
    auth_token = req.get_header('X-AUTH-TOKEN')
    tenant = req.get_header('X-TENANT-ID')

    roles = req.get_header('X-ROLES')
    roles = roles and roles.split(',') or []

    ctxt = context.TVEContext(auth_token=auth_token,
                                  user=user_id,
                                  tenant=tenant,
                                  request_id=request_id,
                                  roles=roles)

    req.env['tveapi.context'] = ctxt
    LOG.info('Request context set')


def before_hooks():
    return [inject_context]

class FuncMiddleware(object):
    """
    Injecting some function as middleware for tveapi
    """
    def __init__(self, func):
        self.func = func

    def process_resource(self, req, resp, resource, params=None):
        return self.func(req, resp, params)
