#!/usr/bin/env python3
import jsonschema
from tveapi.common import json_schemas
from tveapi.common import exceptions

class ServerDoc(object):
    server_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.server_schema)

    @staticmethod
    def validate(doc):
        try:
            ServerDoc.server_doc_validator.validate(doc)
        except Exception as e:
            raise exceptions.BadDataFormat(str(e))

class ConfigServerDoc(object):
    config_server_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.config_server_schema)

    @staticmethod
    def validate(doc):
        try:
            ConfigServerDoc.config_server_doc_validator.validate(doc)
        except Exception as e:
            raise exceptions.BadDataFormat(str(e))

class NetworkDoc(object):
    network_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.networking_schema)

    @staticmethod
    def validate(doc):
        try:
            NetworkDoc.network_doc_validator.validate(doc)
        except Exception as e:
            raise exceptions.BadDataFormat(str(e))

class HomecallingDoc(object):
    homecalling_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.homecalling_schema)

    @staticmethod
    def validate(doc):
        try:
            HomecallingDoc.homecalling_doc_validator.validate(doc)
        except Exception as e:
            raise exceptions.BadDataFormat(str(e))

class InterfacesDoc(object):
    interfaces_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.interfaces_properties_list)

    @staticmethod
    def validate(doc):
        try:
            InterfacesDoc.interfaces_doc_validator.validate(doc)
        except Exception as e:
            raise exceptions.BadDataFormat(str(e))

class InterfaceDoc(object):
    interface_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.interface_properties)

    @staticmethod
    def validate(doc):
        try:
            InterfaceDoc.interface_doc_validator.validate(doc)
        except Exception as e:
            raise exceptions.BadDataFormat(str(e))

class InterfaceDocRequiredAll(object):
    interface_doc_req_all_validator = jsonschema.Draft4Validator(
        schema=json_schemas.interfaces_properties_list_required_all)

    @staticmethod
    def validate(doc):
        try:
            InterfaceDocRequiredAll.interface_doc_req_all_validator.validate(doc)
        except Exception as e:
            raise exceptions.BadDataFormat(str(e))

class NTPDoc(object):
    ntp_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.ntp_schema)

    @staticmethod
    def validate(doc):
        try:
            NTPDoc.ntp_doc_validator.validate(doc)
        except Exception as e:
            raise exceptions.BadDataFormat(str(e))

class HostnameDoc(object):
    hostname_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.hostname_schema)

    @staticmethod
    def validate(doc):
        try:
            HostnameDoc.hostname_doc_validator.validate(doc)
        except Exception as e:
            raise exceptions.BadDataFormat(str(e))

class SecurityAuthDoc(object):
    security_auth_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.security_auth_schema)

    @staticmethod
    def validate(doc):
        try:
            SecurityAuthDoc.security_auth_doc_validator.validate(doc)
        except Exception as e:
            raise exceptions.BadDataFormat(str(e))

class SecurityCliWhitelistDoc(object):
    security_cli_whitelist_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.security_cli_whitelist_schema)

    @staticmethod
    def validate(doc):
        try:
            SecurityCliWhitelistDoc.security_cli_whitelist_doc_validator.validate(doc)
        except Exception as e:
            raise exceptions.BadDataFormat(str(e))

class RemoteServersDoc(object):
    remote_servers_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.remote_server_properties_list)

    @staticmethod
    def validate(doc):
        try:
            RemoteServersDoc.remote_servers_doc_validator.validate(doc)
        except Exception as e:
            raise exceptions.BadDataFormat(str(e))


class SNMPServerDoc(object):
    snmp_server_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.snmp_server_schema)

    @staticmethod
    def validate(doc):
        try:
            SNMPServerDoc.snmp_server_doc_validator.validate(doc)
            for user in doc['users']:
                if user['message-model'] == 'v3':
                    if user['auth-type'] == 'none' or user['priv-type'] == 'none':
                        raise Exception('V3 Users need auth-type and priv-type \
                                                         to be set.')
                if user['message-model'] != 'v3':
                    if user['auth-type'] != 'none' or user['priv-type'] != 'none':
                        raise Exception('V1 and v2c Users do not have \
                            auth-type and priv-type. Please set them to none.')
#             users = []
#             for user in doc['users']:
#                 users.append(user['user-name'])
#
#             for targer in doc['target-address']:
#                 if targer['security-name'] not in users:
#                     raise exceptions.BadDataFormat("{} is not one of ({})".
#                                         format(targer['security-name'], users))
        except Exception as e:
            raise exceptions.BadDataFormat(str(e.message))

class SysResDoc(object):
    sys_res_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.sys_res_schema)

    @staticmethod
    def validate(doc):
        try:
            SysResDoc.sys_res_doc_validator.validate(doc)
        except Exception as e:
            raise exceptions.BadDataFormat(str(e))

class DhcpServerDoc(object):
    dhcp_server_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.dhcp_server_schema)

    @staticmethod
    def validate(doc):
        try:
            DhcpServerDoc.dhcp_server_doc_validator.validate(doc)
        except Exception as e:
            raise exceptions.BadDataFormat(str(e))

class DhcpRelayDoc(object):
    dhcp_relay_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.dhcp_relay_schema)

    @staticmethod
    def validate(doc):
        try:
            DhcpRelayDoc.dhcp_relay_doc_validator.validate(doc)
        except Exception as e:
            raise exceptions.BadDataFormat(str(e))

class ActivationScreenWizardDoc(object):
    activation_screen_wizard_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.activation_screen_wizard_schema)

    @staticmethod
    def validate(doc):
        try:
            ActivationScreenWizardDoc.activation_screen_wizard_doc_validator.validate(doc)
        except Exception as e:
            raise exceptions.BadDataFormat(str(e))

class QuaggaDoc(object):
    quagga_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.quagga_schema)

    @staticmethod
    def validate(doc):
        try:
            QuaggaDoc.quagga_doc_validator.validate(doc)
        except Exception as e:
            raise exceptions.BadDataFormat(str(e))


class QuaggaConfigDoc(object):
    quagga_config_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.quagga_config_schema)

    @staticmethod
    def validate(doc):
        try:
            QuaggaConfigDoc.quagga_config_doc_validator.validate(doc)
        except Exception as e:
            raise exceptions.BadDataFormat(str(e))

class HWTypeDoc(object):
    hardware_type_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.hardware_type_schema)

    @staticmethod
    def validate(doc):
        try:
            HWTypeDoc.hardware_type_doc_validator.validate(doc)
        except Exception as e:
            raise exceptions.BadDataFormat(str(e))

class KubernetesDhcpConfigDoc(object):
    kubernetes_dhcp_config_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.kubernetes_dhcp_config_schema)

    @staticmethod
    def validate(doc):
        try:
            KubernetesDhcpConfigDoc.kubernetes_dhcp_config_doc_validator.validate(doc)
        except Exception as e:
            raise exceptions.BadDataFormat(str(e))

class WireguardClientConfigDoc(object):
    wireguard_client_config_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.wireguard_client_config_schema)
    
    @staticmethod
    def validate(doc):
        try:
            WireguardClientConfigDoc.wireguard_client_config_doc_validator.validate(doc)
        except Exception as e:
            raise exceptions.BadDataFormat(str(e))

class WireguardStaticClientConfigDoc(object):
    wireguard_static_config_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.wireguard_static_client_config_schema)
    
    @staticmethod
    def validate(doc):
        try:
            WireguardClientConfigDoc.wireguard_static_config_doc_validator.validate(doc)
        except Exception as e:
            raise exceptions.BadDataFormat(str(e))
        
class WireguardClientDomainLookupConfigDoc(object):
    wireguard_client_domain_lookup_doc_validator = jsonschema.Draft4Validator(
        schema=json_schemas.wireguard_client_domain_lookup_config_schema)
    
    @staticmethod
    def validate(doc):
        try:
            WireguardClientDomainLookupConfigDoc.wireguard_client_domain_lookup_doc_validator.validate(doc)
        except Exception as e:
            raise exceptions.BadDataFormat(str(e))
