#!/usr/bin/env python3
from tveapi.v1 import config_api
from tveapi.v1 import execute_cli
from tveapi.v1 import heartbeat
from tveapi.v1 import pmstats
from tveapi.v1 import status_api
from tveapi.v1 import tve_license
from tveapi.v1 import wireguard
from tveapi.v1 import wireguard_concentrator
from tveapi.v1.kubernetes import kubernetes
from tveapi.v1.config import files
from tveapi.v1.config import larger_files
from tveapi.v1.config import quagga
from tveapi.v1.config_api import TVE as tve_config_api
from tveapi.v1.status_api import TVE as tve_status_api
from tveapi.v1.kubernetes.kubernetes import TVE as kubernetes_api
from tveapi.v1.kubernetes.images import Image as k8s_image_api
from tveapi.v1.kubernetes.volumes import Volumes as k8s_backup_api
from tveapi.v1 import restore_configuration
from tveapi.v1.config import gpus
from tveapi.v1 import device_details_api

def public_endpoints():
    return [
        # add all config* routes
        ('/config/all', config_api.Config()),

        # ('/config/config-server', tve_config_api.ConfigServer()),

        # ('/config/settings', tve_config_api.Homecalling()),

        # ('/config/networking', tve_config_api.Networking()),

        # ('/config/hostname', tve_config_api.Hostname()),

        ('/config/serial-number', tve_config_api.SerialNumber()),

        ('/config/lct', tve_config_api.LCTConfig()),

        # ('/config/system-resources', tve_config_api.SysRes()),

        # ('/config/dhcp-relay', tve_config_api.DhcpRelay()),

        # ('/config/dhcp-server', tve_config_api.DhcpServer()),

        # ('/config/activation-screen-wizard', tve_config_api.ActivationScreenWizard()),

        # ('/config/networking/interfaces', interfaces.Collection()),

        # ('/config/networking/interfaces/{search}',  interfaces.Item()),

        # ('/config/networking/quagga', quagga.Quagga()),

        # ('/config/networking/quagga/zebra', quagga.Zebra()),

        ('/config/networking/quagga/zebra/file', quagga.ZebraFile()),

        # ('/config/networking/quagga/ospf', quagga.OSPF()),

        ('/config/networking/quagga/ospf/file', quagga.OSPFFile()),

        # ('/config/networking/quagga/bgp', quagga.BGP()),

        ('/config/networking/quagga/bgp/file', quagga.BGPFile()),

        # ('/config/networking/interfaces/{name}', tve_config_api.Interface()),

        # ('/config/hardware/type', tve_config_api.HWType()),

        # ('/config/ntp', tve_config_api.NTP()),

        # ('/config/routing/static', tve_config_api.RoutingStatic()),

        # ('/config/security/authentication', tve_config_api.SecurityAuthentication()),

        # ('/config/security/cli-whitelist', tve_config_api.SecurityCliWhitelist()),

        # ('/config/syslog/remote-servers', tve_config_api.RemoteServers()),

        # ('/config/syslog/remote-servers/{id}', tve_config_api.RemoteServer()),

        # ('/config/snmp-server', tve_config_api.SNMPServer()),

        ('/config/ztp-auth-key', tve_config_api.ZtpAuthUpdate()),

        ('/config/kubernetes-admin-config', tve_config_api.KubernetesAdminConfig()),

        ('/file', files.File()),

        ('/file/large', larger_files.LargerFile()),

        # add all status* routes

        ('/status/settings', tve_status_api.Homecalling()),

        ('/status/networking/interfaces', tve_status_api.NetworkingInterfaces()),

        ('/status/networking/routes', tve_status_api.NetworkingRoutes()),

        ('/status/ntp', tve_status_api.NTP()),

        ('/status/last-known-ztp-error', tve_status_api.ZTP()),

        ('/status/routing/static', tve_status_api.RoutingStatic()),

        ('/status/hardware', tve_status_api.Hardware()),

        ('/status/hardware/cpu', tve_status_api.CPU()),

        ('/status/systemd', tve_status_api.System()),

        ('/status/ucpe', tve_status_api.UCPE()),

        ('/execute-cli', execute_cli.Execute_CLI()),

        ('/restore-configuration', restore_configuration.Restore()),

        ('/reboot', heartbeat.Reboot()),

        ('/apply_config', heartbeat.Apply_Config()),

        ('/send_config', heartbeat.Send_Config()),

        ('/pm_statistics', pmstats.Pm_statistics()),

        ('/ns-status', heartbeat.NS_Status()),

        ('/monitoring', heartbeat.Monitoring()),

        ('/wireguard/public-key', wireguard.PublicKey()),

        ('/wireguard/concentrator-client', wireguard_concentrator.ConcentratorClient()),
        
        ('/wireguard/domain-lookup', wireguard_concentrator.ClientDomainLookup()),

        ('/license/register', tve_license.Register()),

        ('/license/unregister', tve_license.Unregister()),

        ('/status/vm-size', tve_status_api.VMSize()),

        ('/status/br-current', tve_status_api.BackupRestoreCurrent()),
        
        ('/kubernetes/token', kubernetes_api.token()),
        
        ('/kubernetes/worker_status', kubernetes_api.worker_status()),

        ('/kubernetes/worker_status_detailed', kubernetes_api.worker_status_detailed()),
        
        ('/kubernetes/worker_clean', kubernetes_api.worker_clean()),
        
        ('/kubernetes/worker_nodes', kubernetes_api.worker_nodes()),

        # ('/kubernetes/k8s_token', kubernetes_api.k8s_token()),

        # ('/kubernetes/main_vpp_int', kubernetes_api.main_vpp_int())

        ('/kubernetes/image/import', k8s_image_api.Import()),

        ('/kubernetes/image/upload', k8s_image_api.Upload()),

        ('/kubernetes/image/delete', k8s_image_api.Delete()),

        ('/kubernetes/image/download', k8s_image_api.Download()),

        ('/kubernetes/image', k8s_image_api.GetImages()),

        ('/kubernetes/dhcp', kubernetes_api.dhcp()),

        ('/kubernetes/cert', kubernetes_api.cert()),

        ('/kubernetes/backup', k8s_backup_api.Backup()),

        ('/kubernetes/restore', k8s_backup_api.Restore()),

        ('/license/renew', tve_license.Renew()),
        
        ('/license/register_v2', tve_license.Register_v2()),
        
        ('/license/upgrade', tve_license.Upgrade()),

        ('/gpus', gpus.GpuInfoResource()),

        ('/device_details', device_details_api.DeviceDetails())

    ]
