import os
import subprocess
import falcon
from tveapi.common import resource as res
from tveapi import policy
from tveapi.common import utils


class File(res.BaseResource):
    """
    API tests:
    list all files : curl -v -H "X-AUTH-TOKEN: $TOKEN" -H "Content-type:application/json" -k --insecure -X GET https://localhost:7373/v1/file
             - return all files in user folder

    download file : curl -v -H "X-AUTH-TOKEN: $TOKEN" -H "Content-type:application/json" -k --insecure -X GET https://localhost:7373/v1/file?filename=test.sh
             - return file

    upload file : curl -s -H "X-Auth-Token: $TOKEN" -k --insecure -F"file=@test.sh"  https://localhost:7373/v1/file
         - upload file to host
    """
    @policy.enforce('files:get')
    def on_get(self, req, resp):
        resp.status = falcon.HTTP_200
        resp.content_length = 12
        resp.content_type = 'application/octet-stream'
        if not req._params:
            try:
                command = "ls " + res.STORAGE_PATH
                available_files = subprocess.check_output(command, shell=True).decode()
                resp.body = str(available_files)
            except Exception as ex:
                self.set_status_400(resp, 'User folder does not exist')
                return
        else:
            if len(req._params) > 1:
                self.set_status_400(
                    resp, 'Only one search parameter is supported')
                return
            for key, value in req._params.items():
                filename = res.STORAGE_PATH + value
                try:
                    with open(filename, 'rb') as f:
                        resp.body = f.read()
                        f.close()
                    resp.downloadable_as = 'report.pdf'
                except Exception as ex:
                    self.set_status_400(resp, 'File does not exist')
                    return

    @falcon.before(res.BaseResource.file_size(limit=128 * 1024 ** 2))
    def on_post(self, req, resp):
        input_file = req.get_param('file')
        data = input_file.file.read()

        if not os.path.exists(res.STORAGE_PATH):
            utils.execute_shell_command(
                'sudo mkdir {} -m 777'.format(res.STORAGE_PATH))

        if input_file.filename:
            filename = input_file.filename
            file_path = os.path.join(res.STORAGE_PATH, filename)
            temp_file_path = file_path + "~"

            open(temp_file_path, 'wb').write(data)
            os.rename(temp_file_path, file_path)

        resp.body = {'upload': 'ok'}
        resp.status = falcon.HTTP_201
