import falcon
from tveapi.common import resource as res
from tveapi.common import validate_docs as docs
from tveapi.json_tools import json_tools
from tveapi import policy

class Item(res.BaseResource):
    @policy.enforce('config.interface:get')
    def on_get(self, req, resp, search):
        if not req._params:
            self.set_status_400(resp, "Please specify query string.")
        if len(req._params) > 1:
            self.set_status_400(resp, 'Only one search parameter is supported')
            return
        for key, value in req._params.items():
            result = json_tools.get_json_value_by_key(res.UCPE_CONFIG_FILE,
                         '//networking/interfaces', '{}={}'.format(key, value), search)
            resp.body = {search : result}

class Collection(res.BaseResource):
    @policy.enforce('config.interfaces:get')
    def on_get(self, req, resp):
        if not req._params:
            resp.body = json_tools.get_json_value('//networking/interfaces',
                                                   res.UCPE_CONFIG_FILE)
        else:
            if len(req._params) > 1:
                self.set_status_400(resp, 'Only one search parameter is supported')
                return
            for key, value in req._params.items():
                resp.body = json_tools.get_json_value_by_key(res.UCPE_CONFIG_FILE,
                         '//networking/interfaces', '{}={}'.format(key, value))

    @policy.enforce('config.interfaces:put')
    @falcon.before(res.BaseResource.max_body())
    def on_put(self, req, resp):
        doc = self.json_body(req)
        if not req._params:
            docs.InterfaceDocRequiredAll().validate(doc)
        else:
            docs.InterfaceDoc().validate(doc)
        print (doc)

        full_doc = self.get_tve_config_dict()
        if not req._params:
            full_doc['json']['tve']['config']['networking'] = {'interfaces': doc}
        else:
            self.set_status_400(resp, "Changing of single network interface is not supported.")
        self.call_homecalling(resp, full_doc, 'interfaces')
