import falcon
from tveapi.common import resource as res
from tveapi.common import validate_docs as docs
from tveapi.json_tools import json_tools
from tveapi import policy

class Quagga(res.BaseResource):
    @policy.enforce('config.quagga:get')
    def on_get(self, req, resp):
        resp.body = json_tools.get_json_value("//quagga", res.UCPE_CONFIG_FILE)

    @policy.enforce('config.quagga:put')
    @falcon.before(res.BaseResource.max_body())
    def on_put(self, req, resp):
        doc = self.json_body(req)
        docs.QuaggaDoc().validate(doc)

        full_doc = self.get_networking_dict()
        full_doc['json']['tve']['config']['networking'] = {'quagga' : doc }

        self.call_homecalling(resp, full_doc, 'quagga')

class Zebra(res.BaseResource):
    @policy.enforce('config.quagga.zebra:get')
    def on_get(self, req, resp):
        resp.body = json_tools.get_json_value('zebrad-config',res.UCPE_CONFIG_FILE,)

    @policy.enforce('config.quagga.zebra:put')
    @falcon.before(res.BaseResource.max_body())
    def on_put(self, req, resp):
        doc = self.json_body(req)
        docs.QuaggaConfigDoc().validate(doc)

        full_doc = self.get_networking_dict()
        full_doc['json']['tve']['config']['networking']['quagga'] = {'zebrad-config': doc}

        self.call_homecalling(resp, full_doc, 'zebrad-config')


class ZebraFile(res.BaseResource):
    zebra_config_file = '/config/quagga/zebra.conf'

    @policy.enforce('config.quagga.zebra.file:get')
    def on_get(self, req, resp):
        try:
            with open(self.zebra_config_file, 'r') as f:
                content = f.read()
                resp.body = content
        except IOError as ex:
            raise falcon.HTTPError(
                status=falcon.HTTPStatus(404),
                description=ex.message)
        except Exception as ex:
            raise falcon.HTTPError(
                status=falcon.HTTPStatus(400),
                description=ex.message)

class OSPF(res.BaseResource):
    @policy.enforce('config.quagga.ospf:get')
    def on_get(self, req, resp):
        resp.body = json_tools.get_json_value('//ospfd-config', res.UCPE_CONFIG_FILE)

    @policy.enforce('config.quagga.ospf:put')
    @falcon.before(res.BaseResource.max_body())
    def on_put(self, req, resp):
        doc = self.json_body(req)
        docs.QuaggaConfigDoc().validate(doc)

        full_doc = self.get_networking_dict()
        full_doc['json']['tve']['config']['networking']['quagga'] = {'ospfd-config': doc}

        self.call_homecalling(resp, full_doc, 'ospfd-config')


class OSPFFile(res.BaseResource):
    ospf_config_file = '/config/quagga/ospfd.conf'

    @policy.enforce('config.quagga.ospf.file:get')
    def on_get(self, req, resp):
        try:
            with open(self.ospf_config_file, 'r') as f:
                content = f.read()
                resp.body = content
        except IOError as ex:
            raise falcon.HTTPError(
                status=falcon.HTTPStatus(404),
                description=ex.message)
        except Exception as ex:
            raise falcon.HTTPError(
                status=falcon.HTTPStatus(400),
                description=ex.message)

class OSPFv3(res.BaseResource):
    @policy.enforce('config.quagga.ospf6:get')
    def on_get(self, req, resp):
        resp.body = json_tools.get_json_value('//ospf6d-config', res.UCPE_CONFIG_FILE)

    @policy.enforce('config.quagga.ospf6:put')
    @falcon.before(res.BaseResource.max_body())
    def on_put(self, req, resp):
        doc = self.json_body(req)
        docs.QuaggaConfigDoc().validate(doc)

        full_doc = self.get_networking_dict()
        full_doc['json']['tve']['config']['networking']['quagga'] = {'ospf6d-config': doc}

        self.call_homecalling(resp, full_doc, 'ospf6d-config')


class OSPFv3File(res.BaseResource):
    ospf6_config_file = '/config/quagga/ospf6d.conf'

    @policy.enforce('config.quagga.ospf6.file:get')
    def on_get(self, req, resp):
        try:
            with open(self.ospf6_config_file, 'r') as f:
                content = f.read()
                resp.body = content
        except IOError as ex:
            raise falcon.HTTPError(
                status=falcon.HTTPStatus(404),
                description=ex.message)
        except Exception as ex:
            raise falcon.HTTPError(
                status=falcon.HTTPStatus(400),
                description=ex.message)

class BGP(res.BaseResource):
    @policy.enforce('config.quagga.bgp:get')
    def on_get(self, req, resp):
        resp.body = json_tools.get_json_value('//bgpd-config', res.UCPE_CONFIG_FILE)

    @policy.enforce('config.quagga.bgp:put')
    @falcon.before(res.BaseResource.max_body())
    def on_put(self, req, resp):
        doc = self.json_body(req)
        docs.QuaggaConfigDoc().validate(doc)

        full_doc = self.get_networking_dict()
        full_doc['json']['tve']['config']['networking']['quagga'] = {'bgpd-config': doc}

        self.call_homecalling(resp, full_doc, 'bgpd-config')

class BGPFile(res.BaseResource):
    bgp_config_file = '/config/quagga/bgpd.conf'

    @policy.enforce('config.quagga.bgp.file:get')
    def on_get(self, req, resp):
        try:
            with open(self.bgp_config_file, 'r') as f:
                content = f.read()
                resp.body = content
        except IOError as ex:
            raise falcon.HTTPError(
                status=falcon.HTTPStatus(404),
                description=ex.message)
        except Exception as ex:
            raise falcon.HTTPError(
                status=falcon.HTTPStatus(400),
                description=ex.message)
