#!/usr/bin/env python3
import subprocess
import syslog
import psutil
import json
import falcon


def run_command(command, error_message):
    try:
        result = subprocess.check_output(command, shell=True)
        return result.strip().decode('utf-8')  # Assuming the result is a string
    except subprocess.CalledProcessError as ex:
        syslog.syslog(syslog.LOG_ERR, f"{error_message} failed due to: {ex}")
        return 'Unknown'


def get_device_details():
    device_type_command = "sudo dmidecode -t system | grep \"Product Name\" | awk -F: '{print $2}' | tr -d '[:space:]'"
    hw_type_command = 'sudo cat /config/.hwtype'
    serial_number_command = 'sudo cat /config/.serial-number'
    cpu_type_command = "sudo lscpu | awk -F': ' '/Model name/ {gsub(/^[ \\t]+|[ \\t]+$/, \"\", $2); print $2}'"
    online_cpu_cores_command = 'sudo nproc'
    root_disk_size_command = "df -m / | awk 'NR==2 {print $2}'"
    up_time_command = "uptime -p | awk '{$1=\"\"; print $0}'"
    ram_type_command = "sudo dmidecode --type 17 | grep -i 'Form Factor' | awk -F': ' '{print $2}'"

    # psutil commands
    ram_size = psutil.virtual_memory().total // (1024 ** 2)

    device_type = run_command(device_type_command, "Getting the device type")
    hw_type = run_command(hw_type_command, "Getting the hardware type")
    serial_number = run_command(serial_number_command, "Getting the serial number")
    cpu_type = run_command(cpu_type_command, "Getting the CPU type")
    online_cpu_cores = run_command(online_cpu_cores_command, "Getting the number of online CPU cores")
    ram_type = run_command(ram_type_command, "Getting the RAM type")
    root_disk_size = int(run_command(root_disk_size_command, "Getting the root disk size"))
    uptime = run_command(up_time_command, "Getting system uptime")

    device_details = {
            "device_type": device_type,
            "serial_number": serial_number,
            "hardware_type": hw_type,
            "cpu_type": cpu_type,
            "online_cpu_cores": online_cpu_cores,
            "ram_size": ram_size,
            "ram_type": f"{ram_type} RAM",
            "uptime": uptime,
            "root_disk_size": root_disk_size,
        }

    return device_details


class DeviceDetails:

    def on_get(self, req, resp):
        device_info_list = get_device_details()
        resp.body = json.dumps(device_info_list)  # Convert list to JSON string
        resp.content_type = 'application/json'  # Set Content-Type header
        resp.status = falcon.HTTP_200
        return resp.body

