#!/usr/bin/env python3
import json
import multiprocessing
import os
import random
import socket
import subprocess
import threading
import time
from collections import OrderedDict

import falcon
import paramiko
from oslo_log import log
from tveapi.common import resource as res
import sys
sys.path.insert(1, '/usr/lib/python3/dist-packages/tveapi/')
import send_notification
SSH_PORT_FILE = '/config/ssh_port'
# Open the file and read the content

LOG = log.getLogger(__name__)


# Get the ssh port
def get_ssh_port():
    try:
        with open(SSH_PORT_FILE, 'r') as file:
            content = file.read().strip()
            ssh_port = int(content) if content else 22222

    except FileNotFoundError:
        print(f"Error: The file {SSH_PORT_FILE} was not found. Using default port 22222.")
        ssh_port = 22222
    except Exception as e:
        print(f"Error occurred: {e} while getting the ssh-port.")
        ssh_port = 22222

    return ssh_port


# Locally executable CLI cmds
def is_cmd_locally_executable(cmd):
    LOCAL_CMDS = ["ping", "traceroute", "resolve", "show", "system", "iperf", "speed-test", "tcpdump"]
    for iter in LOCAL_CMDS:
        if iter in cmd:
            if "service-control" in cmd:
                return False
            return True
    return False


def is_output_filtered(line, cmd):
    FILTER_STR = ["********************************************", "*     WARNING: Authorized Access Only!     *", "Welcome admin it is", "Welcome tveapi it is",
                  cmd]
    for iter in FILTER_STR:
        if iter in line:
            return True
    return False


# Execute CLIsh cmd locally
class Clish_execute:
    def __init__(self, cmd, timeout, outfile):
        self.timeout = timeout
        self.is_timeout = False
        self.cmd = cmd
        self.outfile_name = outfile
        self.outfile = open(self.outfile_name, 'w+')

    def output_reader(self):
        for line in iter(self.task.stdout.readline, b''):
            line = line.decode('utf-8')
            if is_output_filtered(line, self.cmd):
                continue
            try:
                self.outfile.write(line)
                self.outfile.flush()
                os.fsync(self.outfile)
            except:
                pass

    def error_reader(self):
        for line in iter(self.task.stderr.readline, b''):
            line = line.decode('utf-8')
            if is_output_filtered(line, self.cmd):
                continue
            try:
                self.outfile.write(line)
                self.outfile.flush()
                os.fsync(self.outfile)
            except:
                pass

    def launch(self):
        self.task = subprocess.Popen('/scripts/cli/bin/clish',
                                     stdin=subprocess.PIPE, stdout=subprocess.PIPE,
                                     stderr=subprocess.PIPE, preexec_fn=os.setsid)
        self.t1 = threading.Thread(target=self.output_reader)
        self.t1.start()

        self.t2 = threading.Thread(target=self.error_reader)
        self.t2.start()

        inp = self.cmd + '\n'
        self.task.stdin.write(inp.encode())
        self.task.stdin.close()

    def wait(self):
        for t in range(self.timeout):
            time.sleep(1)
            if self.task.poll() is not None:
                return

        self.is_timeout = True

    def stop(self):
        try:
            self.task.terminate()
        except:
            LOG.debug("Couldn't terminate CLI task.")
        if self.is_timeout == True:
            self.task.terminate()
            self.outfile.close()
        self.t1.join()
        self.t2.join()
        if self.is_timeout == True:
            os.rename(self.outfile_name, self.outfile_name[:-4] + '_timeout.out')
        else:
            self.outfile.close()
            os.rename(self.outfile_name, self.outfile_name[:-4] + '_done.out')


def clish_execute(cmd, outfile_name, password, timeout):
    clish_cmd = Clish_execute(cmd, timeout, outfile_name)
    clish_cmd.launch()
    clish_cmd.wait()
    clish_cmd.stop()
    del clish_cmd


def cli_execute(command, outfile_name, ppassword, timeout):
    LOG.error("cli_execute: command --{}--".format(command))
    LOG.error("cli_execute: outfile -- {} --".format(outfile_name))
    LOG.error("cli_execute: timeout --{}--".format(timeout))
    client = paramiko.SSHClient()
    client.load_system_host_keys()
    ssh_port = get_ssh_port()
    client.set_missing_host_key_policy(paramiko.client.AutoAddPolicy())

    client.connect('127.0.0.1', port=ssh_port, username='tcpdumpuser', key_filename='/home/tcpdumpuser/.ssh/id_rsa', look_for_keys=True)
    is_timeout = False
    start = time.time()
    _, stdout, stderr = client.exec_command(command, bufsize=0, timeout=timeout)
    timediff = int(time.time()) - int(start)
    LOG.error("cli_execute: timediff atstart --{}--".format(timediff))
    with open(outfile_name, 'w+') as f:
        while True:
            try:
                for line in stdout:
                    if line.startswith('Welcome admin it is'):
                        continue

                    f.write(line)
                    f.flush()
                    os.fsync(f)
                    timediff = (int(time.time()) - int(start))
                    LOG.error("cli_execute: timediff stdout {}".format(timediff))
                    if timediff > int(timeout):
                        is_timeout = True
                        break

                for line in stderr:
                    if line.startswith('Welcome admin it is'):
                        continue

                    f.write(line)
                    f.flush()
                    os.fsync(f)
                    timediff = (int(time.time()) - int(start))
                    LOG.error("cli_execute: timediff stderr {}".format(timediff))
                    if timediff > int(timeout):
                        is_timeout = True
                        break

            except socket.timeout:
                is_timeout = True
                break

            if is_timeout == True:
                break

            if True == stdout.channel.exit_status_ready():
                break

            timediff = (int(time.time()) - int(start))
            LOG.error("cli_execute: timediff3 {}".format(timediff))
            if timediff > int(timeout):
                is_timeout = True
                break

    LOG.error("cli_execute: timediff atclose --{}--".format(int(time.time()) - int(start)))
    client.close()
    if is_timeout == True:
        os.rename(outfile_name, outfile_name[:-4] + '_timeout.out')
    else:
        os.rename(outfile_name, outfile_name[:-4] + '_done.out')

    LOG.error("cli_execute: done")


class Execute_CLI(res.BaseResource):
    def on_fail(self, resp, message):
        errmsg = OrderedDict()
        errmsg['404'] = ("{}".format(message))
        resp.body = errmsg
        resp.status = falcon.HTTP_404
        resp.data = json.dumps(errmsg, indent=4, separators=(',', ': ')).encode('utf8')
        resp.content_type = 'application/json'

    def on_get(self, req, resp):
        if not req._params:
            self.on_fail(resp, "file not specified")
            return

        if len(req._params) > 1:
            self.on_fail(resp, "Only one search parameter is supported")
            return

        value = list(req._params.values())[0]
        start_file = "/var/log/tveapi/cli" + value + ".out"
        done_file = "/var/log/tveapi/cli" + value + "_done.out"
        timeout_file = "/var/log/tveapi/cli" + value + "_timeout.out"

        if os.path.exists(done_file):
            resp.status = falcon.HTTP_200
            resp.content_length = 12
            resp.content_type = 'application/octet-stream'
            with open(done_file, 'r') as f:
                fcont = f.read()
                resp.body = fcont

            os.remove(done_file)
        elif os.path.exists(start_file):
            resp.status = falcon.HTTP_206
            resp.content_length = 12
            resp.content_type = 'application/octet-stream'
            with open(start_file, 'r') as f:
                fcont = f.read()
                resp.body = fcont

        elif os.path.exists(timeout_file):
            resp.status = falcon.HTTP_205
            resp.content_length = 12
            resp.content_type = 'application/octet-stream'
            with open(timeout_file, 'r') as f:
                fcont = f.read()
                resp.body = fcont

            os.remove(timeout_file)
        else:
            self.on_fail(resp, "No results found")
            return

        for file in os.listdir('/var/log/tveapi/'):
            if file[-4:].find('.out') >= 0:
                diffs = int(time.time()) - int(os.path.getmtime('/var/log/tveapi/{}'.format(file)))
                if diffs > 9000:
                    # files older that 15mins, delete
                    os.remove('/var/log/tveapi/{}'.format(file))

    #    @falcon.before(res.BaseResource.max_body())
    @falcon.before(send_notification.request_notifier)
    def on_put(self, req, resp):
        json_req = self.json_body(req)
        if 'command' not in json_req:
            self.on_fail(resp, "no command")
            return

        try:
            # no permisions for standard reading !!!
            # tveapi user in sudo group
            passlines = subprocess.check_output(["sudo", "cat", "/root/admin-openrc.env"], stderr=subprocess.STDOUT)
        except subprocess.CalledProcessError as e:
            LOG.error("fail exception CalledProcessError")
            self.on_fail(resp, e)
            return
        except OSError as e:
            LOG.error("fail exception OSError")
            self.on_fail(resp, e)
            return

        pass_list = passlines.decode().split('\n')
        password = pass_list[4][12:]
        ppassword = "-p{}".format(password)
        command = json_req['command']
        #command = 'sudo ' + command
        some_unsecure_int = random.randint(500, 50000)
        outfile_name = "/var/log/tveapi/cli{}.out".format(some_unsecure_int)
        with open(outfile_name, 'w+') as f:
            f.write("")

        timeout = 10
        if 'timeout' in json_req:
            timeout = int(json_req['timeout']) - 4

        if is_cmd_locally_executable(command):
            target = clish_execute

        else:
            target = cli_execute

        command_subprocess_starter = multiprocessing.Process(target=target, args=(command, outfile_name, password, timeout,))
        command_subprocess_starter.daemon = True
        command_subprocess_starter.start()
        resp.body = json.dumps({'commandid': some_unsecure_int}, indent=4, separators=(',', ': ')).encode('utf8')
        resp.status = falcon.HTTP_200
        return
