#!/usr/bin/env python3
import json
from collections import OrderedDict

import falcon
import mysql.connector
from oslo_log import log
from tveapi.common import resource as res

LOG = log.getLogger(__name__)


def get_last_interval_id(cursor):
    sql = "SELECT * FROM state_data"
    cursor.execute(sql)
    records = cursor.fetchall()
    for row in records:
        interval = row[1]
        return interval
    return 0


class Pm_statistics(res.BaseResource):
    def on_fail(self, resp, message):
        errmsg = OrderedDict()
        errmsg['404'] = ("{}".format(message))
        resp.body = errmsg
        resp.status = falcon.HTTP_404
        resp.data = json.dumps(errmsg, indent=4, separators=(',', ': ')).encode('utf8')
        resp.content_type = 'application/json'

    def on_get(self, req, resp):
        try:
            data = {"interval": '', "data": []}
            payload = []
            period = 0
            interval = 0
            if req.content_length > 0:
                json_req = self.json_body(req)
                if 'period' in json_req:
                    period = json_req['period']
                if 'interval' in json_req:
                    interval = json_req['interval']

            mydb = mysql.connector.connect(host='localhost', user="root", passwd="root", database="collectd")
            cursor = mydb.cursor()
            if period == 0:
                if interval == 0:
                    sql = "SELECT * FROM pmstats WHERE interval_id = %s" % get_last_interval_id(cursor)
                elif interval > 0:
                    sql = "SELECT * FROM pmstats WHERE interval_id = %s" % interval
            if period > 0:
                h = period if period in [1, 24] else 1
                m = period if period in [15, 30] else 60
                sql = "SELECT * FROM pmstats WHERE time > unix_timestamp()-(%s*%s*60)" % (h, m)

            cursor.execute(sql)
            records = cursor.fetchall()
            curr_interval = -1
            for row in records:
                if curr_interval == -1:
                    curr_interval = row[0]
                if curr_interval == row[0]:
                    curr_interval = row[0]
                    data['interval'] = row[0]
                    data['data'].append(json.loads(row[2]))
                else:
                    payload.append(data)
                    curr_interval = row[0]
                    data = {"interval": '', "data": []}
                    data['interval'] = row[0]
                    data['data'].append(json.loads(row[2]))

            payload.append(data)
            self.set_response(resp, stderr=None, stdout=json.dumps(payload, indent=4, separators=(',', ': ')).encode('utf8'))
            mydb.close()

        except OSError as e:
            LOG.error("fail exception OSError")
            self.on_fail(resp, e)
            return

        return
