#!/usr/bin/env python3
import json
import os
from collections import OrderedDict

import falcon
from oslo_log import log
from tveapi.common import resource as res
import sys
sys.path.insert(1, '/usr/lib/python3/dist-packages/tveapi/')
import send_notification

LOG = log.getLogger(__name__)

class Restore(res.BaseResource):
    @falcon.before(send_notification.request_notifier)
    def on_fail(self, resp, message):
        errmsg = OrderedDict()
        errmsg['404'] = ("{}".format(message))
        resp.body = errmsg
        resp.status = falcon.HTTP_404
        resp.data = json.dumps(errmsg, indent=4, separators=(',', ': ')).encode('utf8')
        resp.content_type = 'application/json'


    @falcon.before(send_notification.request_notifier)
    def on_put(self, req, resp):
        json_req = self.json_body(req)
        if 'confirm' not in json_req:
            self.on_fail(resp, "no confirmation in the request")
            return

        command = json_req['confirm']

        if "restore-configuration" == command:
            LOG.error("Requested configuration restore")
            token = req.get_header('X-AUTH-TOKEN')
            os.system('/usr/bin/sudo /scripts/cli/tvesh/restore_configuration.sh localhost __INLINE__ "%s"' % token)
            LOG.error("Factory restore is completed")
            resp.body = json.dumps({'commandid': '36895'}, indent=4, separators=(',', ': ')).encode('utf8')
            resp.status = falcon.HTTP_200
            return
        else:
            self.on_fail(resp, "Forbidden operation")
            return
        #
