#!/usr/bin/env python3
import json
import os
import subprocess
from collections import OrderedDict

import falcon
from oslo_log import log
from tveapi.common import resource as res
from tveapi.common import utils
import sys
sys.path.insert(1, '/usr/lib/python3/dist-packages/tveapi/')
import send_notification

LICENSE_FILE_NAME = "ucpe_license.lic"

LOG = log.getLogger(__name__)

# curl -H "X-AUTH-TOKEN:$TOKEN" -k --insecure -X PUT https://<TVE>:7373/v1/license/register -d'{"license_id": "0x01cf"}'
# curl -H "X-AUTH-TOKEN:$TOKEN" -k --insecure -X PUT https://<TVE>:7373/v1/license/unregister
# curl -H "X-Auth-Token: $TOKEN" -k --insecure -F"file=@ucpe_license.lic" https://<TVE>:7373/v1/file

# upload & register/renew/upgrade license file in one go: 
# curl -s -H "X-Auth-Token: $TOKEN" -k --insecure -F"file=@ucpe_license.lic"  https://<TVE>:7373/v1/license/register_v2
# curl -s -H "X-Auth-Token: $TOKEN" -k --insecure -F"file=@ucpe_license.lic"  https://<TVE>:7373/v1/license/renew
# curl -s -H "X-Auth-Token: $TOKEN" -k --insecure -F"file=@ucpe_license.lic"  https://<TVE><TVE>:7373/v1/license/upgrade


class Register(res.BaseResource):
    def on_fail(self, resp, message):
        errmsg = OrderedDict()
        errmsg['404'] = ("{}".format(message))
        resp.body = errmsg
        resp.status = falcon.HTTP_404
        resp.data = json.dumps(errmsg, indent=4, separators=(',', ': ')).encode('utf8')
        resp.content_type = 'application/json'

    def on_get(self, req, resp):
        self.set_status_400(resp, "Only PUT supported.")
        return

    @falcon.before(send_notification.request_notifier)
    def on_put(self, req, resp):
        json_req = self.json_body(req)
        if 'license_id' not in json_req:
            self.on_fail(resp, "No license serial number")
            return

        license_file = os.path.join(res.STORAGE_PATH, LICENSE_FILE_NAME)
        if not input_file.filename:
            resp.body = {'License file not found!'}
            resp.status = falcon.HTTP_404
            return
        
        lic_f = open(license_file,"r")
        json_lic = json.load(lic_f)
        lic_f.close()
        
        LOG.error("json_lic: {}". format(json_lic))
        version = json_lic['license']['version']
        serial = json_lic['license']['serial']
        cpu = json_lic['license']['cpu']
        features = json_lic['license']['features']
        feature_str = ",".join(features) 
        LOG.error("license register version: {}  serial: {}  cpu: {} features: {}".format(version, serial, cpu, feature_str))
        os.system("touch /tmp/license_in_progress")
        license_id = json_req['license_id']
        LOG.critical("tve_license register {}".format(license_id))
        register_ret = subprocess.call("/scripts/cli/tvesh/license_handle.sh register {} {} {}".format(license_id, cpu, feature_str), shell=True)
        LOG.critical("license.py: license_id {} register_ret {} ".format(license_id, register_ret))
        os.system("rm -f /tmp/license_in_progress")
        
        if register_ret == 1:
            resp.body = {'register': 'failed', 'message': 'Serial number not available in license file!'}
            resp.status = falcon.HTTP_201
            return
        elif register_ret == 2:
            resp.body = {'register': 'failed', 'message': 'License file is not available!'}
            resp.status = falcon.HTTP_202
            return
        elif register_ret == 3:
            resp.body = {'register': 'failed', 'message': 'License verification failed!'}
            resp.status = falcon.HTTP_203
            return
        elif register_ret == 4:
            resp.body = {'register': 'failed', 'message': 'License registration failed!'}
            resp.status = falcon.HTTP_203
            return

        resp.body = {'register': 'ok'}
        resp.status = falcon.HTTP_200
        return

class Unregister(res.BaseResource):
    def on_get(self, req, resp):
        self.set_status_400(resp, "Only PUT supported.")
        return
    
    @falcon.before(send_notification.request_notifier)
    def on_put(self, req, resp):
        os.system("touch /tmp/license_in_progress")
        register_ret = subprocess.call(['/scripts/cli/tvesh/license_handle.sh', 'unregister'])
        os.system("rm -f /tmp/license_in_progress")
        resp.body = {'license unregister': 'ok'}
        resp.status = falcon.HTTP_200
        return

class Register_v2(res.BaseResource):
    def on_fail(self, resp, message):
        errmsg = OrderedDict()
        errmsg['404'] = ("{}".format(message))
        resp.body = errmsg
        resp.status = falcon.HTTP_404
        resp.data = json.dumps(errmsg, indent=4, separators=(',', ': ')).encode('utf8')
        resp.content_type = 'application/json'

    def on_get(self, req, resp):
        self.set_status_400(resp, "Only POST supported.")
        return

    def on_put(self, req, resp):
        self.set_status_400(resp, "Only POST supported.")
        return

    @falcon.before(send_notification.request_notifier)
    @falcon.before(res.BaseResource.file_size(limit=128 * 1024 ** 2))
    def on_post(self, req, resp):
        input_file = req.get_param('file')
        data = input_file.file.read()
 
        if not os.path.exists(res.STORAGE_PATH):
            utils.execute_shell_command(
                'sudo mkdir {} -m 777'.format(res.STORAGE_PATH))

        if input_file.filename:
            filename = input_file.filename
            file_path = os.path.join(res.STORAGE_PATH, filename)
            temp_file_path = file_path + "~"

            open(temp_file_path, 'w').write(data.decode())
            os.rename(temp_file_path, file_path)

            resp.body = {'license upload': 'ok'}
            resp.status = falcon.HTTP_201
        
            json_lic = json.loads(data)
            #LOG.error("json_lic: {}". format(json_lic))
            version = json_lic['license']['version']
            serial = json_lic['license']['serial']
            cpu = json_lic['license']['cpu']
            features = json_lic['license']['features']
            feature_str = ",".join(features) 
            #LOG.error("license register version: {}  serial: {}  cpu: {} features: {}".format(version, serial, cpu, feature_str))
            os.system("touch /tmp/license_in_progress")
            LOG.error("register license - serial: {} cpu: {} features: {}".format(serial, cpu, feature_str))
            register_ret = subprocess.call("/scripts/cli/tvesh/license_handle.sh register_v2 {} {} {}".format(serial, cpu, feature_str), shell=True)
            LOG.error("register license: serial {} ret {} ".format(serial, register_ret))
            os.system("rm -f /tmp/license_in_progress")
            resp.body = {'license register': 'ok'}
            resp.status = falcon.HTTP_200
            return
            
        resp.body = {'license register': 'failed'}
        resp.status = falcon.HTTP_400
        return


class Renew(res.BaseResource):
    def on_fail(self, resp, message):
        errmsg = OrderedDict()
        errmsg['404'] = ("{}".format(message))
        resp.body = errmsg
        resp.status = falcon.HTTP_404
        resp.data = json.dumps(errmsg, indent=4, separators=(',', ': ')).encode('utf8')
        resp.content_type = 'application/json'

    def on_get(self, req, resp):
        self.set_status_400(resp, "Only POST supported.")
        return

    def on_put(self, req, resp):
        self.set_status_400(resp, "Only POST supported.")
        return

    @falcon.before(send_notification.request_notifier)
    @falcon.before(res.BaseResource.file_size(limit=128 * 1024 ** 2))
    def on_post(self, req, resp):
        input_file = req.get_param('file')
        data = input_file.file.read()
 
        if not os.path.exists(res.STORAGE_PATH):
            utils.execute_shell_command(
                'sudo mkdir {} -m 777'.format(res.STORAGE_PATH))

        if input_file.filename:
            filename = input_file.filename
            file_path = os.path.join(res.STORAGE_PATH, filename)
            temp_file_path = file_path + "~"

            open(temp_file_path, 'w').write(data.decode())
            os.rename(temp_file_path, file_path)

            resp.body = {'license upload': 'ok'}
            resp.status = falcon.HTTP_201
            #return
        
            json_lic = json.loads(data)
            #LOG.error("json_lic: {}". format(json_lic))
            #version = json_lic['license']['version']
            serial = json_lic['license']['serial']
            cpu = json_lic['license']['cpu']
            features = json_lic['license']['features']
            feature_str = ",".join(features)
            #LOG.error("license renew version: {}  serial: {}  cpu: {} features: {}".format(version, serial, cpu, feature_str))

            LOG.error("renew license - serial: {} cpu: {} features: {}".format(serial, cpu, features))
            register_ret = subprocess.call("/scripts/cli/tvesh/license_handle.sh renew {}".format(serial), shell=True)
            LOG.error("renew license: serial {} ret {} ".format(serial, register_ret))
        
            resp.body = {'license renew': 'ok'}
            resp.status = falcon.HTTP_200
            return
            
        resp.body = {'license renew': 'failed'}
        resp.status = falcon.HTTP_400
        return

class Upgrade(res.BaseResource):
    def on_fail(self, resp, message):
        errmsg = OrderedDict()
        errmsg['404'] = ("{}".format(message))
        resp.body = errmsg
        resp.status = falcon.HTTP_404
        resp.data = json.dumps(errmsg, indent=4, separators=(',', ': ')).encode('utf8')
        resp.content_type = 'application/json'

    def on_get(self, req, resp):
        self.set_status_400(resp, "Only POST supported.")
        return

    def on_put(self, req, resp):
        self.set_status_400(resp, "Only POST supported.")
        return

    @falcon.before(send_notification.request_notifier)
    @falcon.before(res.BaseResource.file_size(limit=128 * 1024 ** 2))
    def on_post(self, req, resp):
        input_file = req.get_param('file')
        data = input_file.file.read()
 
        if not os.path.exists(res.STORAGE_PATH):
            utils.execute_shell_command(
                'sudo mkdir {} -m 777'.format(res.STORAGE_PATH))

        if input_file.filename:
            filename = input_file.filename
            file_path = os.path.join(res.STORAGE_PATH, filename)
            temp_file_path = file_path + "~"

            open(temp_file_path, 'w').write(data.decode())
            os.rename(temp_file_path, file_path)

            resp.body = {'license upload': 'ok'}
            resp.status = falcon.HTTP_201
            #return
        
            json_lic = json.loads(data)
            #LOG.error("json_lic: {}". format(json_lic))
            #version = json_lic['license']['version']
            serial = json_lic['license']['serial']
            cpu = json_lic['license']['cpu']
            features = json_lic['license']['features']
            feature_str = ",".join(features)
            #LOG.error("license upgrade version: {}  serial: {}  cpu: {} features: {}".format(version, serial, cpu, feature_str))

            LOG.error("upgrade license - serial: {} cpu: {} features: {}".format(serial, cpu, features))
            register_ret = subprocess.call("/scripts/cli/tvesh/license_handle.sh upgrade {} {} {}".format(serial, cpu, feature_str), shell=True)
            LOG.error("upgrade license: serial {} ret {} ".format(serial, register_ret))
        
            resp.body = {'license upgrade': 'ok'}
            resp.status = falcon.HTTP_200
            return
            
        resp.body = {'license upgrade': 'failed'}
        resp.status = falcon.HTTP_400
        return
