#!/usr/bin/env python3
import json
import os
from collections import OrderedDict
from tveapi.common import validate_docs as docs
import falcon
from oslo_log import log
from tveapi.common import resource as res
import sys
from tveapi.common import utils

import edgility_db.edgility_db as edgedb
import ucpeif.wireguardvpnif as wg

sys.path.insert(1, '/usr/lib/python3/dist-packages/tveapi/')

LOG = log.getLogger(__name__)

class ConcentratorClient(res.BaseResource):
    def on_fail(self, resp, message):
        errmsg = OrderedDict()
        errmsg['404'] = ("{}".format(message))
        resp.body = errmsg
        resp.status = falcon.HTTP_404
        resp.data = json.dumps(errmsg, indent=4, separators=(',', ': ')).encode('utf8')
        resp.content_type = 'application/json'
    
    def on_put(self, req, resp):
        doc = self.json_body(req)
        docs.WireguardClientConfigDoc().validate(doc)

        hostname = doc['hostname']
        password = doc['password']
        public_key = doc['public-key']
        conc_port = doc['concentrator-port']
        endpoint = req.remote_addr

        code, stdout, stderr = utils.execute_shell_command(
            f"/usr/bin/sudo python3 /scripts/set_wireguard_concentrator.py dynamic {hostname} {password} {public_key} {conc_port} {endpoint}")

        resp.body = json.loads(stdout)
        resp.status = falcon.HTTP_200
        resp.content_type = 'application/json'

class WireguardStaticClient(res.BaseResource):
    def on_fail(self, resp, message):
        errmsg = OrderedDict()
        errmsg['404'] = ("{}".format(message))
        resp.body = errmsg
        resp.status = falcon.HTTP_404
        resp.data = json.dumps(errmsg, indent=4, separators=(',', ': ')).encode('utf8')
        resp.content_type = 'application/json'

    def on_post(self, req, resp):
        doc = self.json_body(req)
        docs.WireguardStaticClientConfigDoc().validate(doc)

        db = edgedb.EdgilityDatabase()

        hostname = doc['hostname']
        tunnel_ip = doc['tunnel-ip']

        tables = db.get_all_tables('wireguard')
        for table in tables:
            if table.startswith('dynamic_clients_'):
                ips = db.fetch_column_entries(table, 'tunnel_ip')
                if tunnel_ip in ips:
                    self.on_fail(resp, 'The requested IP address is already in use')
                    return

        try:
            db.insert_values_in_table()
        except Exception:
            pass
        
class ClientDomainLookup(res.BaseResource):
    def on_fail(self, resp, message):
        errmsg = OrderedDict()
        errmsg['404'] = ("{}".format(message))
        resp.body = errmsg
        resp.status = falcon.HTTP_404
        resp.data = json.dumps(errmsg, indent=4, separators=(',', ': ')).encode('utf8')
        resp.content_type = 'application/json'
    
    def on_post(self, req, resp):
        doc = self.json_body(req)
        docs.WireguardClientDomainLookupConfigDoc().validate(doc)

        hostname = doc['hostname']
        domain_lookup = doc['domain-lookup']

        code, stdout, stderr = utils.execute_shell_command(
            f"/usr/bin/sudo python3 /scripts/set_wireguard_concentrator.py domain_lookup {hostname} {domain_lookup}")

        LOG.error(f"stdout{stdout}stdout")
        if code != 0:
                self.set_status_500(resp, stderr, stdout)
        else:
            resp.body = {'execute': 'ok'}
            resp.status = falcon.HTTP_201
        return

        
